%  This is a demo file illustrating the usage of hierarchical ICI-base CDT
%
%
% Giacomo Boracchi
% Politecnico di Milano
% July 2014
% giacomo.boracchi@polimi.it
%
% References
%
% [1]  Cesare Alippi, Giacomo Boracchi and Manuel Roveri,
% "A just-in-time adaptive classification system based on the intersection of confidence intervals rule,"
% Neural Networks, Elsevier vol. 24 (2011), pp. 791-800 doi: 10.1016/j.neunet.2011.05.012
%
% [2] Cesare Alippi, Giacomo Boracchi and Manuel Roveri,
% "A Hierarchical, Nonparametric Sequential Change-Detection Test"
% in Proceedings of IJCNN 2011, the International Joint Conference on Neural Networks,
% San Jose, California July 31 - August 5, 2011. pp 2889 - 2896, DOI: 10.1109/IJCNN.2011.6033600

close all
clear
clc

% randn('seed', 2)
% rand('seed', 2)

addpath('Functions/')
addpath('SyntheticDatasets/')

%% generate dataset

AMPLITUDES= [2000, 2000]; % T^* is at time instant 2000, and then there are other 2000 samples in the new state
pdfStr = 'Gaussian';
pdfParams = [0 1; 0.5 1;]; % an abrupt change of 0.5 * \sigma to the classes mean
% pdfParams = [0 1; 0 2;]; % an abrupt change which doubles the variance 
m = 0;
do_show = 1;
figureNumber = 12;

DataSet = f_creaDataSetMultiChanges_CDT(AMPLITUDES, pdfStr , pdfParams , m , do_show , figureNumber);

%% run the ICI-based CDT

tsLength = 400;
% generate dataset parameters
cdtParams = define_ICI_test_parameters(tsLength);

%run the hierarchical ICI-based CDT.
[tsEnd, tsInit, confirmedDetections, tsLengths , testDetectingChange] = f_ICI_test_MultiChange_MultiStage(DataSet, tsLength , cdtParams);

disp(['Detections of the ICI-based CDT at the first layer: ', num2str(unique(confirmedDetections))])
disp(['   (these detections have been validated by the second layer)']);
disp(['Detections of the CDT ', num2str(unique(tsEnd(2 : end)))]);
disp(['Refined estimate of the change-time instant ', num2str(unique(tsInit(2 : end)))]);

%% shows the results
LN_WDT = 2;
figure(1),
plot(DataSet , 'r.');
hold on
a = get(gca);
range = a.YLim;
for ii = 2 : length(tsInit)
    plot([tsInit(ii) , tsInit(ii)] , range , 'g--' , 'LineWidth' , LN_WDT);
    plot([tsEnd(ii) , tsEnd(ii)] , range , 'b--' , 'LineWidth' , LN_WDT);
    if (length(confirmedDetections) >= ii)
        plot([confirmedDetections(ii) , confirmedDetections(ii)] , range , 'k--' , 'LineWidth' , LN_WDT);
    end
end
for ii = 2 : numel(AMPLITUDES)
       plot([AMPLITUDES(ii) , AMPLITUDES(ii)] , range , 'm-' , 'LineWidth' , LN_WDT);

end
legend('data', 'T_{ref}' , 'T_{end}' ,  'First Layer Detections', 'True change');
hold off
xlabel('time')
ylabel('observations')





